#include <d3dx10.h>
#include <dxerr.h>
#include "BaseMesh.h"
#include "MeshModel.h"
#include "SceneManagement/Elements/Camera.h"
#include "ResourceManagement/Material/Material.h"
#include "Log/Logger.h"
#include "Core/MarabuntaCore.h"

using namespace Marabunta::Mesh;
using namespace Marabunta::SceneManagement;
using namespace Marabunta::Render;
using namespace Marabunta;

/**
 * BaseMesh is an abstract class for rendering any kind of mesh. 
 * It stores a MeshModel which holds the static mesh information.
 * BaseMesh adds a transformation matrix, locators, bounding box info.
 *
 * It implements functionality for preparing the mesh like setting
 * its layout abnd its material.
 *
 * It also implements the preparation for rendering like setting the
 * effect variables and the camera. The render call is also in this class.
 *
 * BaseMesh is extended by classes like StaticMesh and Animated mesh.
 *
 * @author		Matthew Bozarth
 * @author		Josh Gilpatrick
 * @updated		4/20/2009
 */

/**
 * Constructor, it's abstract so it's not implemented.
 */
//BaseMesh::BaseMesh(void)
//{
//}

/**
 * This constructor is called from the inheriting classes and is used
 * for setting up the material,  saving the device, and initializing the
 * effect variables.
 */
BaseMesh::BaseMesh(Marabunta::Render::CompiledMaterial* mat)
{
	_material = mat;
	_device = MarabuntaCore::Instance()->GetDevice()->GetDevice();
	SetupEffectVariables();
}

/**
 * Deconstructor
 */
BaseMesh::~BaseMesh(void)
{
}

/**
 * Replaces the current material with a new one. 
 * This lets us re-use a geometry with new material properties.
 * For example, we can make a mesh look damaged by changing the material.
 *
 * @param mat the new material
 */
void BaseMesh::SetMaterial(CompiledMaterial* mat)
{
	//is it the same effect? if so, don't re-map the variables
	if (_material->Effect == mat->Effect)
	{
		_material = mat;
	}
	else
	{
		_material = mat;
		SetupEffectVariables();
	}
}

/**
 * Creates the layout for Geometry. Layouts are instance specific.
 *
 * @return true if it worked, false otherwise.
 */
bool BaseMesh::CreateLayout()
{
	HRESULT hr;

	// Create the vertex input layout
	D3D10_PASS_DESC PassDesc;
	_material->Technique->GetPassByIndex(0)->GetDesc(&PassDesc);

	hr = _device->CreateInputLayout(
		_mesh->VertexLayout,
		_mesh->NumElements, 
		PassDesc.pIAInputSignature, 
		PassDesc.IAInputSignatureSize,
		&_mesh->InputLayout);

	if(FAILED(hr))
	{
		CRITICAL_LOG << "DXERROR: " << DXGetErrorDescription(hr) << std::endl;
		return false;
	}
	return true;
}

/**
 * Maps the resource variables in C++ to the HLSL shader variables.
 * Must be called if the shader changes.
 * Extended in sub-classes if it needs more.
 */
void BaseMesh::SetupEffectVariables()
{
	_worldMatrix = _material->Effect->GetVariableByName("World")->AsMatrix();
	_viewMatrix = _material->Effect->GetVariableByName("View")->AsMatrix();
	_projectionMatrix = _material->Effect->GetVariableByName("Projection")->AsMatrix();
}

/**
 * Sets effect variables in anticipation of rendering.
 * View and projection are not unique to a BaseMesh, so this only sets world.
 *
 * @param world the new world matrix
 */
void BaseMesh::SetEffectVariables(D3DXMATRIX* world)
{
	_worldMatrix->SetMatrix(*world);
}

/**
 * Sets the layout before we render. 
 * Called for each mesh because other meshes could change it.
 */
void BaseMesh::SetLayout()
{
	_device->IASetInputLayout(_mesh->InputLayout);
	// Set primitive topology
	_device->IASetPrimitiveTopology(D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST);
}

/**
 * Sets the camera variables: projection and view.
 *
 * @param camera this contains the view and projection matrices
 */
void BaseMesh::SetCamera(Camera* cam)
{
	_projectionMatrix->SetMatrix((float*)cam->projectionMatrix);
	_viewMatrix->SetMatrix((float*)cam->viewMatrix);
}

/**
 * Renders the mesh with all of the previously set states.
 * Sets the vertex buffer, index buffer, and then loops over the technique passes.
 */
void BaseMesh::Render()
{
	//set the vertex buffer
	_device->IASetVertexBuffers(0, 1, &_mesh->VertexBuffer, &_mesh->Stride, &_mesh->Offset);
	//set the index buffer
	_device->IASetIndexBuffer(_mesh->IndexBuffer, DXGI_FORMAT_R32_UINT, 0);
	
	for (UINT p = 0; p < _material->TechniqueDescription.Passes; p++)
	{
		_material->Technique->GetPassByIndex(p)->Apply(0); //the argument isn't used
		_device->DrawIndexed(_mesh->NumberOfIndices, 0, 0);
	}
}