#include "MeshManager.h"
#include "../../Utilities/Utilities.h"
#include "StaticMesh/StaticMesh.h"
#include "AnimatedMesh/AnimatedMesh.h"
#include "Core/MarabuntaCore.h"
#include "ResourceManagement/Mesh/MeshModel.h"


using namespace Marabunta::Mesh;
using namespace Marabunta;

/**
 * Responsible for creating and dispensing of meshes.
 *
 * @author		Josh Gilpatrick
 * @updated		4/20/2009
 */

/**
 * Constructor.
 */
MeshManager::MeshManager()
{
	this->_device = MarabuntaCore::Instance()->GetDevice()->GetDevice();
}

/**
 * Deconstructor.
 * Deletes and properly releases all video card resources being used.
 */
MeshManager::~MeshManager(void)
{
	for(UINT i = 0; i < _staticModels.size();i++)
	{
		ReleaseCOM(_staticModels[i].Mesh->IndexBuffer);
		ReleaseCOM(_staticModels[i].Mesh->VertexBuffer);
		ReleaseCOM(_staticModels[i].Mesh->InputLayout);
		SafeDelete(_staticModels[i].Mesh);	
		SafeDelete(_staticModels[i].StaticMesh->LocatorNodes);
		SafeDelete(_staticModels[i].StaticMesh);
	}
	for(UINT i = 0; i < _animatedModels.size();i++)
	{
		ReleaseCOM(_animatedModels[i].Mesh->IndexBuffer);
		ReleaseCOM(_animatedModels[i].Mesh->VertexBuffer);
		ReleaseCOM(_animatedModels[i].Mesh->InputLayout);
		SafeDelete(_animatedModels[i].Mesh);	
		for(UINT j = 0; j < _animatedModels[i].AnimatedMesh->Header.NumberOfJoints;j++)
		{
			SafeDelete(_animatedModels[i].AnimatedMesh->Joints[j].Frames);
		}
		SafeDelete(_animatedModels[i].AnimatedMesh->Joints);

		SafeDelete(_animatedModels[i].AnimatedMesh->LocatorNodes);
		SafeDelete(_animatedModels[i].AnimatedMesh);
	}
	CleanUp();
}

/**
 * Creates an animated mesh based on the geometry parameters.
 *
 * @param geom - info describing the mesh to be built
 * @param mesh - the output mesh
 */
void MeshManager::CreateMesh(GeometryParameters geom, AnimatedMesh** mesh)
{
	*mesh = new AnimatedMesh(geom);
	_animatedMeshes.push_back(*mesh);

	for(UINT i = 0; i < _animatedModels.size();i++)
	{
		if(_animatedModels[i].Name == geom.MeshName)
		{
			(*mesh)->Initialize(_animatedModels[i].AnimatedMesh,_animatedModels[i].Mesh);
			return;
		}
	}
	AnimatedMeshResource mr;
	mr.Name = geom.MeshName;
	mr.Mesh = new MeshModel();
	mr.AnimatedMesh = new TFAnimatedMesh();
	AnimatedMesh::LoadMesh(_device,"./Meshes/" + geom.MeshName,&mr.Mesh,&mr.AnimatedMesh);
	(*mesh)->Initialize(mr.AnimatedMesh,mr.Mesh);
	_animatedModels.push_back(mr);
}

/**
 * Creates a static mesh based on the geometry parameters.
 *
 * @param geom - info describing the mesh to be built
 * @param mesh - the output mesh
 */
void MeshManager::CreateMesh(GeometryParameters geom, StaticMesh** mesh)
{
	*mesh = new StaticMesh(geom);
	_staticMeshes.push_back(*mesh);

	for(UINT i = 0; i < _staticModels.size();i++)
	{
		if(_staticModels[i].Name == geom.MeshName)
		{
			(*mesh)->Initialize(_staticModels[i].StaticMesh,_staticModels[i].Mesh);
			return;
		}
	}
	StaticMeshResource mr;
	mr.Name = geom.MeshName;
	mr.Mesh = new MeshModel();
	mr.StaticMesh = new TFStaticMesh();
	StaticMesh::LoadMesh(_device,"./Meshes/" + geom.MeshName,&mr.Mesh,&mr.StaticMesh);
	(*mesh)->Initialize(mr.StaticMesh,mr.Mesh);
	_staticModels.push_back(mr);
}

/**
 * Deletes all of the meshes from their respective vectors
 */
void MeshManager::CleanUp()
{
	for(UINT i = 0; i < _animatedMeshes.size();i++)
	{
			SafeDelete(_animatedMeshes[i]);
	}
	for(UINT i = 0; i < _staticMeshes.size();i++)
	{
			SafeDelete(_staticMeshes[i]);
	}
}