#include "StaticMesh.h"
#include "Utilities/Utilities.h"
#include "Log/Logger.h"
#include "Core/MarabuntaCore.h"
#include "ResourceManagement/Shaders/VertexStructs.h"
#include "ResourceManagement/Mesh/MeshModel.h"
#include "SceneManagement/Elements/BoundingVolume/SphereBV.h"
#include "ResourceManagement/Material/MaterialManager.h"

using namespace Marabunta::Mesh;
using namespace Marabunta::Render;

/**
 * StaticMesh extends BaseMesh and is not abstract. It is used to render actual
 * instances of static meshes. 
 *
 * New fields: 
 * a texture
 *
 * New functions: 
 * loading and setting up static meshes
 * effect variable modification functions overwritten to support textures
 *
 * @author		Josh Gilpatrick
 * @author		Matthew Bozarth
 * @updated		4/25/2009
 */

StaticMesh::StaticMesh(GeometryParameters geom) : BaseMesh(geom.Material)
{
}

/**
 * Deconstructor
 */
StaticMesh::~StaticMesh()
{
	
}

/**
 * Initializes the mesh.
 * Sets the bounding volumes, sets locators, creates the input layout and binds effect variables
 *
 * @param mesh - static mesh specific info
 * @param model - generic mesh info
 */
void StaticMesh::Initialize(TFStaticMesh* mesh,MeshModel* model)
{
	_mesh = model;
	
	_meshBoundingBox = mesh->BoundingBox;

	for(int i = 0; i < mesh->Header.NumberOfLocators;i++)
	{
		_locators.push_back(mesh->LocatorNodes[i].GlobalTranslation);
	}
	CreateLayout();
	SetupEffectVariables();
}

/**
 * Replaces the current material with a new one. 
 * This lets us re-use a geometry with new material properties.
 * For example, we can make a mesh look damaged by just changing the material.
 *
 * I expect calling "SetupEffectVariables" is not quick.
 *
 * @param mat the new material
 */
void StaticMesh::SetMaterial(CompiledMaterial* mat)
{
	//is it the same effect? if so, don't re-map the variables
	if (_material->Effect == mat->Effect)
	{
		_material = mat;
	}
	else
	{
		_material = mat;
		SetupEffectVariables();
		BaseMesh::SetupEffectVariables();
	}
}

/**
 * Maps the resource variables in C++ to the HLSL shader variables.
 * Must be called if the shader changes.
 * Adds a texture in addition to the standard basemesh variables.
 */
void StaticMesh::SetupEffectVariables()
{
	_textureResource = _material->Effect->GetVariableByName("baseTexture")->AsShaderResource();
}

/**
 * Sets effect variables in anticipation of rendering.
 * Overwrites the basemesh function to add a texture.
 */
void StaticMesh::SetEffectVariables(D3DXMATRIX* world)
{
	_textureResource->SetResource(_material->Texture);
	BaseMesh::SetEffectVariables(world);
}

/**
 * Loads the mesh
 *
 * @param device - graphics device
 * @param fileName - name of the mesh file
 * @param model - general model info, vertex buffer, etc
 * @param mesh - model info specific to the static mesh
 */
void StaticMesh::LoadMesh(ID3D10Device* device,std::string fileName, MeshModel** model, TFStaticMesh** mesh)
{

	FILE* file = NULL;
	size_t items = 0;
	TFHeader header;
	TFStaticVertex* vertices;
	UINT* indices;

	errno_t err = fopen_s(&file,fileName.c_str(),"rb");
	if(file == NULL)
	{
		ERROR_LOG << "File not found: " << fileName << endl;
		return;
	}
	//items = fread_s(&header,sizeof(TFHeader),sizeof(TFHeader),1,file);
	items = fread(&header,sizeof(TFHeader),1,file);
	if(items != 1)
	{
		ERROR_LOG << "Header incorrect file: " << fileName << endl;
		return;
	}
	if(header.MagicNumber != MAGIC_NUMBER)
	{
		ERROR_LOG << "Magic number mismatch: " << fileName << ":" <<  header.MagicNumber << endl;
		return;
	}
	if(header.Version != MESH_VERSION)
	{
		ERROR_LOG << "Version number mismatch: "  << fileName << ":" << header.Version << endl;
		return;
	}

	items = fread(&(*mesh)->Header,sizeof(TFStaticHeader),1,file);
	if(items != 1)
	{
		ERROR_LOG << "Header could not be read: " << fileName << endl;
		return;
	}
	
	vertices = new TFStaticVertex[(*mesh)->Header.NumberOfVertices];
	indices = new UINT[(*mesh)->Header.NumberOfIndices];
	(*mesh)->LocatorNodes = new TFLocatorNode[(*mesh)->Header.NumberOfLocators];
	items = fread(vertices,sizeof(TFStaticVertex),(*mesh)->Header.NumberOfVertices,file);
	if(items != (*mesh)->Header.NumberOfVertices)
	{
		ERROR_LOG << "Verticies could not be read: " << fileName << endl;
		return;
	}

	items = fread(indices,sizeof(UINT),(*mesh)->Header.NumberOfIndices,file);
	if(items != (*mesh)->Header.NumberOfIndices)
	{
		ERROR_LOG << "indices could not be read: " << fileName << endl;
		return;
	}

	items = fread(&(*mesh)->BoundingBox,sizeof(TFBoundingBox),1,file);
	if(items != 1)
	{
		ERROR_LOG << "Could not read bounding box" << endl;
	}

	items = fread((*mesh)->LocatorNodes,sizeof(TFLocatorNode),(*mesh)->Header.NumberOfLocators,file);
	if(items != (*mesh)->Header.NumberOfLocators)
	{
		ERROR_LOG << "Could not read locator nodes" << endl;
	}

	fclose(file);

	SetupMesh(device,model,mesh,&vertices,&indices);

	delete vertices;
	delete indices;
}

/**
 * Sets up the resources for the mesh.
 * Builds vertex and index buffers
 *
 * @param device - graphics device
 * @param model - generic model info, vertex buffer, etc
 * @param mesh - static mesh specific info
 * @param vertices - static mesh vertex info
 * @param indices - indices
 */
void StaticMesh::SetupMesh(ID3D10Device* device,MeshModel** model, TFStaticMesh** mesh, TFStaticVertex** vertices, UINT** indices)
{
	//Set parent variables 
	(*model)->Stride = sizeof(TFStaticVertex);
	(*model)->NumberOfIndices = (*mesh)->Header.NumberOfIndices;
	(*model)->VertexLayout = VertexPositionNormalTextureTIndex_Layout;
	(*model)->NumElements = sizeof(VertexPositionNormalTextureTIndex_Layout)/sizeof(VertexPositionNormalTextureTIndex_Layout[0]);

	D3D10_BUFFER_DESC bd;
	D3D10_SUBRESOURCE_DATA InitData;
	int size = sizeof(**vertices);

	//SET VERTEX BUFFER
	bd.Usage = D3D10_USAGE_DEFAULT;
	bd.ByteWidth = (*model)->Stride * (*mesh)->Header.NumberOfVertices;
    bd.BindFlags = D3D10_BIND_VERTEX_BUFFER;
	bd.CPUAccessFlags = 0;
    bd.MiscFlags = 0;

    InitData.pSysMem = *vertices;
	device->CreateBuffer( &bd, &InitData, &(*model)->VertexBuffer );

	//SET INDEX BUFFER
    bd.Usage = D3D10_USAGE_DEFAULT;
	bd.ByteWidth = sizeof( UINT ) * (*mesh)->Header.NumberOfIndices;
    bd.BindFlags = D3D10_BIND_INDEX_BUFFER;
    bd.CPUAccessFlags = 0;
    bd.MiscFlags = 0;

    InitData.pSysMem = *indices;
    device->CreateBuffer( &bd, &InitData, &(*model)->IndexBuffer );
}