#include "Camera.h"
#include "Utilities/Utilities.h"
#include "Core/MarabuntaCore.h"

using namespace Marabunta::SceneManagement;

/**
 * A camera. Multiple cameras can exist in a scene, but only one can be active.
 * Camera contains a spatial, but it also contains view and projection
 * matrices. These are built using the look at and up vectors.
 *
 * The idea is that the camera can be set to look at a point in spac
 * or it can be attached to a spatial, and look at that.
 *
 * @author		Matthew bozarth
 * @updated		4/25/2009
 */


const float Camera::NEAR_CLIP_PLANE = 0.1f;
const float Camera::FAR_CLIP_PLANE = 5000.f;

/**
 * Constructor, instantiates the view and projection matrices 
 * and calls CreateCamera().
 */
Camera::Camera(void)
{

	viewMatrix = new D3DXMATRIX();
	projectionMatrix = new D3DXMATRIX();
	CreateCamera();
}

/**
 * Deconstructor, deletes the view matrix and the projection matrix.
 * Deletes its children.
 */
Camera::~Camera(void)
{
	SafeDelete(viewMatrix);
	SafeDelete(projectionMatrix);

	if(!children.empty())
	{
		std::vector<Spatial*>::iterator it;
		it = children.begin();
		while(it != children.end())
		{
			SafeDelete(*it);
			it++;
		}
		children.clear();
	}
}

/**
 * Initializes the camera with the default values:
 * - position of (0,0,10)
 * - lookat at the origin
 * - up vector of Y
 *
 * Adds the up spatial and the lookat spatial to the children list.
 * These must be in indices 0 and 1.
 */
void Camera::CreateCamera()
{
	if (children.size() != 0)
	{
		ERROR_LOG << "Camera has children, this is going to prevent it from working correctly. Do not set camera children!" << std::endl;
		return;
	}

	SetTranslation(D3DXVECTOR3(0,0,-10));

	Spatial* t;

	//set Up - this hass to be first
	t = new Spatial();
	t->SetTranslation(D3DXVECTOR3(0,0,1));
	AddChild(t);
	
	//set LookAt - this has to be 2nd
	t = new Spatial();
	t->SetTranslation(D3DXVECTOR3(0,0,0));
	AddChild(t);

	//set LookAt target to be null and off
	pLookAtTarget = NULL;	
	useLookAtTarget = false;

	//build the view matrix for the first time
	UpdateGS(0);

	BuildProjectionMatrix();
}

/**
 * Updates the camera's position by calling the super method.
 * Builds the view matrix using the look at spatial and the up spatial.
 * Should be called once a frame.
 *
 * @param dt time since the last update
 */
void Camera::UpdateGS(float dt)
{
	Node::UpdateGS(dt);
	BuildViewMatrix();
}

/**
 * Builds the view matrix for the camera based on its position,
 * look at, and up vectors.
 */
void Camera::BuildViewMatrix()
{
	//move the camera
	position.x = _world._41;
	position.y = _world._42;
	position.z = _world._43;

	//get the look at, depending on our state
	if (useLookAtTarget)
		pLookAtTarget->GetTranslation(lookat);
	else
		this->children[LOOK_AT]->GetTranslation(lookat);

	//set the Up vector to be the location of the Up node
	up = children[UP]->GetWorldTranslation();
	//subtract our position from it to get a directional vector
	up -= position;

	//build the matrix
	D3DXMatrixLookAtLH(viewMatrix, &position, &lookat, &up);
}

/**
 * Builds the camera's projection matrix based on the FOV,
 * aspect ratio, near and far clip planes.
 */
void Camera::BuildProjectionMatrix()
{
	D3DXVECTOR4 window = MarabuntaCore::Instance()->GetWindow()->GetWindowRect();
	fov.y = (float)D3DX_PI * .25f /* window.w/window.z*/;
	fov.x = fov.y * window.z/window.w;
	D3DXMatrixPerspectiveFovLH(projectionMatrix, fov.y, window.z/window.w, NEAR_CLIP_PLANE, FAR_CLIP_PLANE);
}

/**
 * Tells the camera to look at a fixed position.
 * Switches the state of the camera to be looking at its fixed lookat.
 *
 * @param a The position it will look at.
 */
void Camera::SetLookAt(D3DXVECTOR3 at)
{
	children[LOOK_AT]->SetTranslation(at);
	useLookAtTarget = false;
}

/**
 * Tells the camera to look at a Spatial.
 * Useful if you want the camera to follow a moving object.
 * Changes the state of the camera to be looking at its target spatial.
 * If the input spatial is 0, it will activate look at target instead of setting it.
 *
 * @param p The Spatial it will look at.
 */
void Camera::SetLookAt(Spatial* s)
{
	if (s == 0)
	{	
		useLookAtTarget = true;
		return;
	}
	pLookAtTarget = s;
	useLookAtTarget = true;
}

/**
 * Sets the up vector for the camera.
 * @param up vector3
 */
void Camera::SetUp(D3DXVECTOR3 up)
{
	children[UP]->SetTranslation(up);
}
/**
 * Moves the point that the camera looks at.
 * Will cause the camera to use the look at point if it's not already;
 * @param delta vector3 describing how the look at should move
 */
void Camera::TranslateLookAt(D3DXVECTOR3 delta)
{
	children[LOOK_AT]->Translate(delta);
	useLookAtTarget = false;
}

/**
 * Returns the point the camera is looking at
 */
D3DXVECTOR3 Camera::GetLookAt()
{
	return lookat;
}

/**
 * @return vector3 of up
 */
D3DXVECTOR3 Camera::GetUp()
{
	return up;
}

/**
 * @return the field of view of this camera
 */
D3DXVECTOR2 Camera::GetFOV()
{
	return fov;
}
