
#include "Geometry.h"
#include "Core/MarabuntaCore.h"
#include <dxerr.h>
#include "ResourceManagement/Material/Material.h"
#include "BoundingVolume/BoundingVolume.h"
#include "BoundingVolume/SphereBV.h"
#include "BoundingVolume/AxisAlignedBV.h"
#include "BoundingVolume/OrientedBoxBV.h"

#pragma warning (disable:4482) // nonstandard enum type space warning 

using namespace Marabunta;
using namespace Marabunta::SceneManagement;
using namespace Marabunta::Render;

/**
 * A scene graph type. Geometry extends Spatial and adds a renderable component
 * and a render function. As Geometry is a Spatial, it cannot have children.
 *
 * @author		Matthew Bozarth
 * @updated		4/20/2009
 */

/**
 * Constructor, calls the super constructor.
 * Takes in a BaseMesh that is either of StatichMesh or AnimatedMesh type.
 */
Geometry::Geometry(Mesh::BaseMesh* bm) : Spatial()
{
	_type = ElementType::ET_Geometry;

	_mesh = bm;
	_boundingVolume->ContainAABB(_mesh->_meshBoundingBox.Min, _mesh->_meshBoundingBox.Max);
	_boundingVolume->_drawable = true;

	//do this so the BV values are initialized.
	UpdateBS();

	SetupEffectVariables();
}

/**
 * Deconstructor.
 */
Geometry::~Geometry(void)
{
	SafeDelete(_boundingVolume);
}

/**
 * Replaces the current material with a new one. 
 * This lets us re-use a geometry with new material properties.
 * For example, we can make a mesh look damaged by just changing the material.
 *
 * @param mat the new material
 */
void Geometry::SetMaterial(CompiledMaterial* mat)
{
	_mesh->SetMaterial(mat);
}

/**
 * Maps the resource variables in C++ to the HLSL shader variables.
 * Must be called if the shader changes.
 */
void Geometry::SetupEffectVariables()
{
	_mesh->SetupEffectVariables();
}

/**
 * Sets effect variables in anticipation of rendering.
 */
void Geometry::SetEffectVariables()
{
	_mesh->SetEffectVariables(&_world);
}

/**
 * Sets the layout in anticipation of rendering
 */
void Geometry::SetLayout()
{
	_mesh->SetLayout();
}

/**
 * Tells the mesh to set the camera.
 * @param cam - the camera being used
 */
void Geometry::SetCamera(Camera* cam)
{
	_mesh->SetCamera(cam);
}

/**
 * Passes the drawable and any of its children to the 
 * render graph so it can be drawn
 */
void Geometry::UpdateDraw()
{
	Marabunta::Render::RenderGraph::AddRenderable(this);
}

/**
 * Renders the element.
 */
void Geometry::Render()
{
	_mesh->Render();
}

/**
 * @return the geometry's mesh
 */
BaseMesh* Geometry::GetMesh()
{
	return _mesh;
}

/**
 * Sets the Geometry's mesh
 * @param m - the new mesh
 */
void Geometry::SetMesh(Mesh::BaseMesh* m)
{
	_mesh = m;
}