#include "Node.h"
#include "Utilities/Utilities.h"
#include "Geometry.h"
#include "BoundingVolume/BoundingVolume.h"
#include "BoundingVolume/SphereBV.h"
#include "BoundingVolume/AxisAlignedBV.h"
#include "BoundingVolume/OrientedBoxBV.h"
#include "SceneManagement/Graphs/SpatialGraph.h"

using namespace Marabunta::SceneManagement;

/**
 * Node is extended from Spatial and adds children.
 * A node's bounding volume encompasses the bounding 
 * volumes of all of its children.
 *
 * @author		Matthew Bozarth
 * @updated		4/25/2009
 */


/**
 * Constructor, calls the super constructor.
 * Sets its type as ET_Node.
 */
Node::Node(void) : Spatial()
{
	_type = ET_Node;
}

/**
 * Deconstructor
 * Destroys its children.
 */
Node::~Node(void)
{
	if(!children.empty())
	{
		int size = (int)children.size();
		for(int i = 0; i < size; ++i)
		{
			SafeDelete(children[i]);
		}
	}
}

/**
 * Adds a new spatial as a child.
 *
 * @param child - the spatial being added to the children list
 */
void Node::AddChild(Spatial* child)
{
	child->SetParent(this);
	children.push_back(child);

	//if child has a BV
	if (child->GetBound() != 0)
	{
		//and the parent doesn't
		if (!_boundingVolume)
		{
			_boundingVolume = BoundingVolume::Create();
			_boundingVolume->CopyFrom(child->GetBound());
			_boundingVolume->_drawable = true;
		}
		else //parent already does 
		{
			_boundingVolume->GrowToContain(child->GetBound());
			_boundingVolume->_drawable = true;
		}
	}
}

/**
 * Attempts to remove a spatial from 
 * the Node's children list, if it exists.
 *
 * @param child the Spatial that will be removed
 */
void Node::RemoveChild(Spatial* child)
{
	std::vector<Spatial*>::iterator itr = children.begin();
	while(itr != children.end())
	{
		if((*itr) == child)
		{
			child->SetParent(0);
			children.erase(itr);
			break;
		}
		++itr;
	}
}

/**
 * Flags a child node for removal to avoid lock stepping with threading
 * There is one list for each thread that is trying to remove elements
 * 
 * @param child - the element to be removed
 * @param listID - id relating to the thread removing it
 */
void Node::FlagForRemoval(Spatial* child, unsigned int listID)
{
	//if thread n is trying to add, make sure we have least n lists
	while(_removalList.size() < (listID + 1))
		_removalList.push_back(std::vector<Spatial*>());

	_removalList[listID].push_back(child);
}

/**
 * Updates its matrices as well as the matrices of all its children.
 *
 * @param dt time in float since the last update
 */
void Node::UpdateGS(float dt)
{
	//remove any children that are flagged for it
	//it's threadsafe to do it here
	for(size_t i = 0; i < _removalList.size(); ++i)
	{
		for(size_t j = 0; j < _removalList[i].size(); ++j)
		{
			RemoveChild(_removalList[i][j]);
		}
		_removalList[i].clear();
	}

	//call the parent UpdateGS
	Spatial::UpdateGS(dt);

	//update children's GS
	size_t size = children.size();
	for (UINT i = 0; i < size; i++)
	{
		children[i]->UpdateGS(dt);			
	}

	//if you don't have a BV, then don't update your children's BV
	//node should ALWAYS have a BV if its children do.
	if (_boundingVolume)
	{
		for (UINT i = 0; i < size; i++)
		{
			BoundingVolume* bv = children[i]->UpdateBS();
			//children[i]->UpdateGS(dt);			
			_boundingVolume->GrowToContain(bv);
		}
	}
}

/**
 * Update's the Node's bounding state. Just calls the spatial function.
 */
BoundingVolume* Node::UpdateBS()
{
	return Spatial::UpdateBS();
}

/**
 * Passes the drawable and any of its children to the 
 * render graph so it can be drawn
 */
void Node::UpdateDraw()
{
	size_t size = children.size();
	for (UINT i = 0; i < size; i++)
	{
		children[i]->UpdateDraw();
	}
}