#include "Spatial.h"
#include "Geometry.h"
#include "../Graphs/RenderGraph.h"
#include "SceneManagement/Scene.h"
#include "Log/Logger.h"
#include "BoundingVolume/BoundingVolume.h"
#include "BoundingVolume/SphereBV.h"
#include "BoundingVolume/AxisAlignedBV.h"
#include "BoundingVolume/OrientedBoxBV.h"

#include "Utilities/Utilities.h"

#pragma warning (disable:4482) // nonstandard enum type space warning 

using namespace Marabunta::SceneManagement;

/**
 * Spatial is the base element of a scene graph. 
 * It has a transformation matrix, boundign volume and a 
 * parent spatial but no children.
 * 
 * @author		Matthew Bozarth
 * @updated		4/25/2009
 */


/**
 * Constructor, calls Initialize().
 */
Spatial::Spatial(void)
{
	_boundingVolume = 0;
	_parent = 0;
	_boundingVolume = BoundingVolume::Create();
	Initialize();
}

/**
 * Deconstructor, does nothing.
 */
Spatial::~Spatial(void)
{
	SafeDelete(_boundingVolume);
}

/**
 * Initializes the following members:
 * - scale: (1,1,1)
 * - rotation: (0,0,0)
 * - translation: (0,0,0)
 * - localTransform: Identity
 * - derivedTransform: Identity
 *
 * Builds the local transform matrices and 
 * sets the dirty flag to false.
 */
void Spatial::Initialize()
{
	_type = ElementType::ET_Spatial;
	_scale = D3DXVECTOR3(1,1,1);
	_rotation = D3DXVECTOR3(0,0,0);
	_translation = D3DXVECTOR3(0.0f,0.0f,0.0f);

	_local = D3DXMATRIX();
	_parent = 0;

	
	UpdateGS(0);
	UpdateBS();
}

/**
 * Sets the parent of the spatial.
 *
 * @param p the parent spatial
 */
void Spatial::SetParent(Spatial* p)
{
	_parent = p;
}

#pragma region accessors
/**
 * Adds the input vector to the spatial's translation.
 *
 * @param delta The vector3 added.
 */
void Spatial::Translate(D3DXVECTOR3 delta)
{
	_translation += delta;
	_dirtyTransform = true;
}

/**
 * Modifies the translation of the spatial to be the input position.
 *
 * @param newPos - the new position 
 */
void Spatial::SetTranslation(D3DXVECTOR3 newPos)
{
	_translation = newPos;
	_dirtyTransform = true;
}

/**
 * Returns the translation as an out variable.
 *
 * @param &outPos - the current position 
 */
void Spatial::GetTranslation(D3DXVECTOR3 &outPos)
{
	outPos = _translation;
}

/**
 * Returns the local translation.
 *
 * @return the local location as vector3
 */
D3DXVECTOR3 Spatial::GetTranslation()
{
	return _translation;
}

/**
 * Adds the input vector to the spatial's rotation.
 *
 * @param delta The vector3 added.
 */
void Spatial::Rotate(D3DXVECTOR3 delta)
{
	_rotation += delta;
	_dirtyTransform = true;
}

/**
 * Modifies the rotation of the spatial to be the input rotation.
 *
 * @param newRot - the new rotation 
 */
void Spatial::SetRotation(D3DXVECTOR3 newRot)
{
	_rotation = newRot;
	_dirtyTransform = true;
}

/**
 * Returns the rotation as an out variable.
 *
 * @param  &outRot - the current rotation 
 */
void Spatial::GetRotation(D3DXVECTOR3 &outRot)
{
	outRot = _rotation;
}

/**
 * Returns the rotation.
 *
 * @return vector3 of the rotation
 */
D3DXVECTOR3 Spatial::GetRotation()
{
	return _rotation;
}

/**
 * Adds the input vector to the spatial's scale.
 *
 * @param delta The vector added.
 */
void Spatial::Scale(D3DXVECTOR3 delta)
{
	_scale += delta;
	_dirtyTransform = true;
}

/**
 * Modifies the scale of the spatial to be the input scale.
 *
 * @param newScale - the new scale as a vector3
 */
void Spatial::SetScale(D3DXVECTOR3 newScale)
{
	_scale = newScale;
	_dirtyTransform = true;
}

/**
 * Returns the scale as an out variable.
 *
 * @param &outScale - the current scale 
 */
void Spatial::GetScale(D3DXVECTOR3 &outScale)
{
	outScale = _scale;
}

/**
 * Returns the scale.
 *
 * @return the scale as a vector3
 */
D3DXVECTOR3 Spatial::GetScale()
{
	return _scale;
}
#pragma endregion

/**
 * Returns the derived transform. Use this for rendering.
 *
 * @return matrix of the world transformation
 */
D3DXMATRIX* Spatial::GetWorld()
{
	if (_dirtyTransform) BuildLocal();
	return &_world;
}

/**
 * Returns the derived translation vector.
 *
 * @return D3DXVECTOR3 of the derived position
 */
D3DXVECTOR3 Spatial::GetWorldTranslation()
{
	if (_dirtyTransform) BuildLocal();
	D3DXVECTOR3 out;
	out.x = _world._41;
	out.y = _world._42;
	out.z = _world._43;
	return out;
}

/**
 * If the dirty transform flag is set, it will generate new 
 * scaling, rotation, and translation matrices and then set the flag to false
 */
void Spatial::BuildLocal()
{
	//doesn't currently work with up-down hierarchy traversal
	//if (!_dirtyTransform) return;

	D3DXMatrixScaling(&_scaleTemp, _scale.x, _scale.y, _scale.z);
	D3DXMatrixRotationYawPitchRoll(&_rotationTemp, _rotation.y, _rotation.x, _rotation.z);
	D3DXMatrixTranslation(&_translationTemp, _translation.x, _translation.y, _translation.z);
	
	D3DXMatrixIdentity(&_local); //identity
	D3DXMatrixMultiply(&_local, &_local, &_scaleTemp); //scale
	D3DXMatrixMultiply(&_local, &_local, &_rotationTemp); //rotation
	D3DXMatrixMultiply(&_local, &_local, &_translationTemp); //translation

	if (_parent)
		D3DXMatrixMultiply(&_world, &_local, _parent->GetWorld());
	else
		_world = _local;

	_dirtyTransform = false;
}



/**
 * Updates the geometry state of the spatial based on time.
 * Calls BuildLocalTransform to generate a new transformation matrix.
 * If it has a parent, will set the derived transform to be the combine of 
 * local and parent.
 * If unparented, will set derived to be the local transform.
 *
 * @param dt time since the last update
 */
void Spatial::UpdateGS(float dt)
{
	BuildLocal();
}

/**
 * Updates the bounding state of the spatial if it has a BV.
 * Will send its bounding volume up the scene graph.
 *
 * @return this spatial's bounding volume
 */
BoundingVolume* Spatial::UpdateBS()
{
	if (_boundingVolume)
	{
		UpdateWorldBound();
		PropagateBoundToRoot();
		return this->_boundingVolume;
	}
}

/**
 * Passes the drawable and any of its children to the 
 * render graph to be drawn.
 *
 * The spatial version does nothing.
 */
void Spatial::UpdateDraw() {}

/**
 * Updates the world matrix of the bounding volume.
 */
void Spatial::UpdateWorldBound()
{
	if (_boundingVolume)
		_boundingVolume->TransformBy(this->_world);
}

/**
 * If this spatial has a parent, 
 * will force the parent to update its BV
 */
void Spatial::PropagateBoundToRoot()
{
	if (_parent)
	{
		_parent->UpdateWorldBound();
		_parent->PropagateBoundToRoot();
	}
}

/**
 * @return the spatial's bounding volume
 */
BoundingVolume* Spatial::GetBound()
{
	return _boundingVolume;
}

/**
 * @return whether or not the spatial's transform has been 
 * updated and not yet packed into a matrix
 */
bool Spatial::IsDirty()
{
	return _dirtyTransform;
}