#include "RenderGraph.h"
#include "SceneManagement/Elements/Camera.h"
#include "SceneManagement/Elements/Geometry.h"
#include "SceneManagement/Elements/Spatial.h"
#include "SceneManagement/Elements/BoundingVolume/BoundingVolume.h"
#include "Core/MarabuntaCore.h"

using namespace Marabunta::Render;
using namespace Marabunta::SceneManagement;

/**
 * Responsible for rendering World Objects.
 * It is a static class that contains a list of things to render. 
 * This list is generated and cleared every frame. 
 * This allows for easy sorting and batching of elements if desired.
 *
 * The render graph can render wireframes of bounding volumes and 
 * supplies a console function "RenderBV()" for toggling this in game.
 *
 * @author		Matthew Bozarth
 * @updated		4/20/2009
 */

std::vector<Geometry*> RenderGraph::renderables;

bool RenderGraph::bvRender = true;

/** 
 * Constructor
 */
RenderGraph::RenderGraph(void)
{
	
}

/** 
 * Deconstructor
 */
RenderGraph::~RenderGraph(void)
{
}

/**
 * Sets up the Lua functions.
 */
void RenderGraph::Initialize()
{
	lua_State* L = MarabuntaCore::Instance()->GetLuaState();
	RegisterLuaFunctions(L);
}

/**
* Renders all of the renderables.
* After rendering, clears its list for the next pass.
*
* @param cam supplies the camera specific settings for rnedering the scene
*/
void RenderGraph::Render(Camera* cam)
{
	//-per render rule
	if (renderables.size() > 0)
	{
		//render each geometry
		UINT size = (UINT)renderables.size();
		for (UINT i = 0; i < size; i++)
		{
			renderables[i]->SetCamera(cam);
			renderables[i]->SetLayout();
			renderables[i]->SetEffectVariables();
			renderables[i]->Render();
		}

		//for drawing bounding volumes
		if (bvRender)
		{
			//draw the bounding volumes in wireframe
			Marabunta::MarabuntaCore::Instance()->GetDevice()->SetFillMode(D3D10_FILL_WIREFRAME);
			D3DXMATRIX* boundWorld;
			StaticMesh* sm;
			size = (UINT)renderables.size();
			for (UINT i = 0; i < size; i++)
			{
				if (renderables[i]->GetBound()->_drawable)
				{
					boundWorld = renderables[i]->GetBound()->GetWorld();
					sm = renderables[i]->GetBound()->GetWireFrame();
					sm->SetCamera(cam);
					sm->SetLayout();
					sm->SetEffectVariables(boundWorld);
					sm->Render();
				}
			}
			Marabunta::MarabuntaCore::Instance()->GetDevice()->SetFillMode(D3D10_FILL_MODE::D3D10_FILL_SOLID);
		}
	}

	//clear it out in preparation for the next frame
	renderables.clear();
}

/**
* Adds geometry to the renderable list for this frame.
*
* @param g The geometry to be added.
*/
void RenderGraph::AddRenderable(Geometry* g)
{
	renderables.push_back(g);
}

/**
 * Lua hook that toggles the drawign of wireframes. 
 */
int RenderGraph::RenderBV(lua_State* L)
{
	bvRender = !bvRender;
	return 0;
}

/**
 * Tells the Lua state about the functions defined in this class.
 */
void RenderGraph::RegisterLuaFunctions(lua_State* L)
{
	lua_getglobal(L, "M");
	lua_pushcfunction(L, RenderBV);
	lua_setfield(L, -2, "RenderBV");
	lua_pop(L, 1);
}