#include "SceneGraph.h"
#include "../Elements/Node.h"
#include "../Elements/Spatial.h"
#include "../Elements/Camera.h"
#include "../Elements/Geometry.h"
#include "Utilities/Utilities.h"
#include "SpatialGraph.h"

using namespace Marabunta::SceneManagement;

/**
 * Responsible for hierarchical animation. Anything that needs to be updated
 * should go into this graph. Update happens in a top down fashion. 
 * A root node at the top is updated and it updates all of its children
 * in a recursive fashion. 
 * (it might be interesting to write an iterative traversal)
 *
 * There are three basic elements that go into the scene graph:
 * -spatial
 * -node
 * -geometry
 * -camera
 * -world object
 *
 * Spatial is the base class and contains information about its relative
 * transformation, stores this into a local transformation matrix
 * and is able to multiply that by its parent to build its world matrix.
 * While spatials have parents, and can thus exist in the tree, they
 * do not have children and can only be leaf nodes. Spatials also have a 
 * bounding volume, which by default should take up no space and can be
 * turned on or off.
 *
 * Node extends Spatial and adds children. Its update function updates
 * itself and its children; both their geometric and bound states.
 *
 * Geometry extends Spatial (it can be only be a child) and adds a Render() 
 * function as well as a BaseMesh field. This allows it to be drawable.
 *
 * Camera extends spatial and does a few unique things. It ds several
 * camera specific fields: view matrix, projection matrix, up, lookat
 * as well as providing functions for generating and retrieving these values.
 * A scene graph may have many cameras and it treats them all the same.
 * When a scene is draw though, it can only have one active camera.
 * A camera has a couple child spatials. These are to store the up and 
 * look at vectors. This way if a camera is rotated, its up and look at
 * vectors are updated automatically through the generic scene graph update.
 *
 * World Object extends node, and can thus have children. It doesn't add very
 * much; just drawable, collidable booleans and an id. Its importance is in 
 * that other systems in the engine work with world objects, such as the 
 * spatial graph and the render graph. 
 *
 * Update involves two things, updating the geometric state (UpdateGS)
 * and updating the bound state (UpdateBS). UpdateGS updates the 
 * transformation matrices of each spatial, while UpdateBS sets new values
 * for the bounding volume. The scene graph doesn't really know much 
 * about the game and only knows about spatials, nodes, and geometry. This
 * allows it to be used for many different things. We extend Node to create
 * a WorldObject that can be inserted into the RenderGraph and its children
 * are rendered. WorldObject is meant to be extended by the game application.
 * If you want to make a world object that can be rendered, as you often do,
 * then give it a geometry child.
 *
 * @author		Matthew Bozarth
 * @updated		4/20/2009
 */

/**
 * Constructor. Builds the root node.
 */
SceneGraph::SceneGraph(void)
{
	root = new Node();
}

/**
 * Deconstructor.
 * Deletes the node, which will go ahead and delete all of its children.
 * Do not leave anything in the scene graph that you want to delete yourself.
 * Please remove all elements that will be cleaned up by managers before quitting.
 */
SceneGraph::~SceneGraph(void)
{
	SafeDelete(root);
}

/**
 * Updates the entire scene graph, 
 * building the matrices for anything that has been changed.
 *
 * @param dt time since the last update
 */
void SceneGraph::Update(float dt)
{
	root->UpdateGS(dt);
	root->UpdateBS();
}

/**
 * @return the root node.
 */
Node* SceneGraph::Root()
{
	return root;
}