#include "SpatialGraph.h"
#include "../Elements/Node.h"
#include "../Elements/Spatial.h"
#include "../Elements/WorldObject.h"
#include "RenderGraph.h"
#include "SceneManagement/Elements/BoundingVolume/BoundingVolume.h"

using namespace Marabunta::SceneManagement;

/**
 * SpatialGraph is responsible for frustum culling and providing collision
 * testing between objects. It also passes objects on to the render graph.
 *
 * The world object list is maintained and updated. Other systems must manually 
 * add and remove world objects from the list. Update, depending on the flags for
 * the world object, will update its status within the collision system, and
 * will add the object to the render graph to be rendered.
 *
 * When the spatial graph loops over the world objects, 
 * it tests to see if each object is within the frustum, and only passes
 * the objects that pass to the geometry list.
 *
 * It is also responsible for testing intersections and will internally optimize
 * how this is done. Not currently implemented, but I expect I will use 
 * bit-binning.
 *
 * @author		Matthew Bozarth
 * @updated		4/25/2009
 */

/**
 * Constructor
 */
SpatialGraph::SpatialGraph()
{

}

/**
 * Deconstructor
 */
SpatialGraph::~SpatialGraph()
{

}

/**
 * Updates the elements within the collision system 
 * (if they are flagged for collision)
 * If they are flagged for rendering, add them to the render graph.
 */
void SpatialGraph::Update()
{
	for(size_t i = 0; i < _removalList.size(); ++i)
	{
		for(size_t j = 0; j < _removalList[i].size(); ++j)
		{
			RemoveWorldObject(_removalList[i][j]);
		}

		_removalList[i].clear();
	}

	//update all of the world objects in the bin system
	//TODO

	//update all of the world object geometry children
	int size = _worldObjects.size();
	for (int i = 0; i < size; i++)
	{
		//if it is marked to be drawable
		if (_worldObjects[i]->_drawable)
		{
			//if passes frustum culling
			//TODO
			{
				_worldObjects[i]->UpdateDraw();
			}
		}

		//if it is marked to be collidable
		if (_worldObjects[i]->_collidable)
		{
			if (_worldObjects[i]->IsDirty())
			{
				//update its position in the collision system
			}
		}
	}
}

/**
 * Adds objects to the world object list. These objects are put into the 
 * collision graph and can be collided against.
 */
void SpatialGraph::AddWorldObject(WorldObject* n)
{
	_worldObjects.push_back(n);
}

/**
 * Removes a world object from the list of collidables
 * @param the object we are removing
 */
void SpatialGraph::RemoveWorldObject(WorldObject* n)
{
	std::vector<WorldObject*>::iterator iter = _worldObjects.begin();
	for (; iter != _worldObjects.end(); iter++)
	{
		if (*iter == n)
		{
			_worldObjects.erase(iter);
			return;
		}
	}
}

/**
 * Flags a world object for removal to avoid lock stepping with threading
 * There is one list for each thread that is trying to remove elements
 * 
 * @param n - the object being removed
 * @param listID - id relating to the thread removing it
 */
void SpatialGraph::FlagForRemoval(WorldObject* n, unsigned int listID)
{
	while(_removalList.size() < (listID + 1))
		_removalList.push_back(std::vector<WorldObject*>());

	_removalList[listID].push_back(n);
}

/**
* Looks to see if the spatial passed in collides with anything. 
* If it collided with at least one element, returns true.
* Terminates early
* O(n) worst case, doesn't allocate much memory
* Recommend you use this if you only care if your object collided
* Do not use as a pre-test before you use FindCollisions
* While this will be faster if it returns false, if it returns true
* you are now traversing the list twice.
*
* @param in the spatial we are testing for collision
* @return true if someone collided, false if nothing collided
*/
bool SpatialGraph::FindCollision(BoundingVolume* inBV)
{
	size_t size = _worldObjects.size();
	BoundingVolume* bv;
	for (size_t i = 0; i < size; i++)
	{
		bv = _worldObjects[i]->GetBound();

		//it does't collide
		if (!bv)
			continue;

		//it's the same as the input BV
		if(inBV == bv)
			continue;

		//if it collides, return true
		if (inBV->TestIntersection(bv))
		{
			return true;
		}
	}
	return false;
}

/**
 * Looks to see if the spatial passed in collides with anything. 
 * Anything it collides with is added to a list 
 * and this list is then passed out.
 *
 * O(n) all cases, allocates N memory
 *
 * @param in the spatial we are testing for collision
 * @return true if someone collided, false if nothing collided
 */
std::vector<WorldObject*> SpatialGraph::FindCollisionList(BoundingVolume* inBV)
{
	size_t size = _worldObjects.size();
	std::vector<WorldObject*> out;

	BoundingVolume* bv;
	for (int i = 0; i < size; i++)
	{
		//it's flagged to be not collidable
		if (!_worldObjects[i]->_collidable)
			continue;

		bv = _worldObjects[i]->GetBound();
		
		//it has no BV
		if (!bv)
			continue;

		//it's the same as the input BV
		if(inBV == bv)
			continue;

		//if it does collide, add it!
		if (inBV->TestIntersection(bv))
		{
			out.push_back(_worldObjects[i]);
		}
	}
	return out;
}