#include "Scene.h"
#include "SceneManagement/Graphs/SceneGraph.h"
#include "SceneManagement/Graphs/SpatialGraph.h"
#include "SceneManagement/Graphs/RenderGraph.h"
#include "SceneManagement/Terrain/Terrain.h"
#include "Log/Logger.h"
#include "Utilities/Utilities.h"

using namespace Marabunta::SceneManagement;

/**
 * A scene is a combination of a scene graph and a spatial graph.
 * The scene graph is responsible for creating, updating and
 * maintaining a list of all objects in the scene.
 * The spatial graph is used to do collision between elements 
 * within the scene and doing frustum culling.
 *
 * After the scene graph has been updated and the spatial graph
 * has been updated, the list is passed to the render graph,
 * which is not part of the scene as render graph is completely static.
 *
 * The scene graph and the spatial graph are separate.
 * If something should be updated, put it in the scene graph.
 * If something should be collided against or rendered, put it in the
 * spatial graph. If it needs both, then put it in both. There is no
 * automation transfer of WorldObjects from the scenegraph to the 
 * spatial graph.
 *
 * @author Matthew Bozarth
 * @updated		4/20/2009
 */

/**
 * Constructor
 * Makes a default scene.
 * Creates the scene graph, the spatial graph and defaults to inactive.
 *
 * @param name sets the name of the scene
 */
Scene::Scene(std::string name)
{
	pSceneGraph = new SceneGraph();
	pSpatialGraph = new SpatialGraph();
	pTerrain = NULL;
	active = false;
	this->name = name;

	pCurrentCamera = 0;
}

/**
 * Deconstructor
 * deletes the scene graph, spatial graph, and the terrain.
 */
Scene::~Scene(void)
{
	SafeDelete(pSceneGraph);
	SafeDelete(pSpatialGraph);
	SafeDelete(pTerrain);
}

/**
 * Updates the scene, updating its scene graph and spatial graph.
 * @param dt time since last update
 */
void Scene::Update(float dt)
{
	pSceneGraph->Update(dt);
	//elements get passed from the scene to the spatial graph here
	//via black voodoo (or during their update call). 
	//ie any world objects you want to be collided,
	//must be added to the spatial graph manually.
	pSpatialGraph->Update();
}

/**
 * Renders the scene by calling the render graph.
 */
void Scene::Render()
{
	if( pTerrain != NULL ) pTerrain->Render(pCurrentCamera);
	Marabunta::Render::RenderGraph::Render(pCurrentCamera);
}

/**
 * Activates the scene, setting its active bool to true.
 */
void Scene::Activate()
{
	active = true;
}

/**
 * Dectivates the scene, setting its active bool to false.
 */
void Scene::Deactivate()
{
	active = false;
}

/**
 * @return the scene's name.
 */
std::string& Scene::Name()
{
	return name;
}